/*********************************************************************
 *
 *                Microchip USB C18 Firmware Version 1.0
 *
 *********************************************************************
 * FileName:        user.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company) for its PICmicro Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Rawin Rojvanit       11/19/04    Original.
 ********************************************************************/

/******************************************************************************
 * CDC RS-232 Emulation Tutorial Instructions:
 ******************************************************************************
 * Refer to Application Note AN956 for explanation of the CDC class.
 * 
 * First take a look at Exercise_Example() and study how functions are called.
 * 
 * There are five exercises, each one has a solution in the CDC\user\solutions.
 * Scroll down and look for Exercise_01,_02,_03,_04, and _05.
 * Instructions on what to do is inside each function.
 * 
 *****************************************************************************/

/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include <usart.h>
#include "system\typedefs.h"
#include "system\usb\usb.h"
#include "io_cfg.h"             // I/O pin mapping
#include "user\user.h"
#include <stdlib.h>
#include "usb_send.h"
#include "delay_s.h"
#include "eeprom.h"

#include <pwm.h>
#include <timers.h>

/** V A R I A B L E S ********************************************************/
#pragma udata
unsigned char input_buffer[64];
unsigned char output_buffer[64];
//unsigned int counter;


/** P R I V A T E  P R O T O T Y P E S ***************************************/
void BlinkUSBStatus(void);
void User_Process(void);
//void SendADC(void);
//void ReadPOT(void);

#pragma code
void UserInit(void)
{
mInitAllLEDs()
TRISA 			 = 0x00;
TRISAbits.TRISA0 = 1;
//OpenADC(ADC_FOSC_RC&ADC_LEFT_JUST&ADC_0_TAD,ADC_CH0&ADC_INT_OFF&ADC_REF_VDD_VREFMINUS&ADC_VREFMINUS_VDD,14);
ADCON0 = 0b00000000;
ADCON1 = 0b00001110;
ADCON2 = 0b10000111;

ADCON0bits.ADON = 1;
//SetChanADC(ADC_CH0);

TRISC = 0x00;
TRISD = 0x00;
TRISB = 0xFF;
INTCON2bits.RBPU=0;

//PWM period =[(period ) + 1] x 4 x TOSC x TMR2 prescaler



OpenTimer2(TIMER_INT_OFF & T2_PS_1_4 & T2_POST_1_1);

// 91		32,967.032 kHz
// 83		36,144.578 kHz
// 79		37,974.683 kHz
// 53		57,692.307 kHz

OpenPWM1(83); // IR
SetDCPWM1(0);

OpenPWM2(83); // AM
SetDCPWM2(0);

}//end UserInit

/******************************************************************************
 * Function:        void ProcessIO(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user routines.
 *                  It is a mixture of both USB and non-USB tasks.
 *
 * Note:            None
 *****************************************************************************/
void ProcessIO(void)
{   
    BlinkUSBStatus();
    // User Application USB tasks
    if((usb_device_state < CONFIGURED_STATE)||(UCONbits.SUSPND==1)) return;

    User_Process();

}//end ProcessIO


#define IR_166us	140
#define IR_277us	238

#define IR_416us	361
#define IR_444us 	413

#define IR_611us 	542
#define IR_777us 	727

#define IR_1ms		901
#define IR_350us 	320
#define IR_500us 	450

#define IR_PWMDC_OM	256 //256 //25% 512 // 50%

#define AM_PWMDC_OM	512 //256 //25% 512 // 50%


char runad = 0;
char runioi = 0;
unsigned int result = 0;
unsigned char i = 0;
unsigned char a = 0;
unsigned char b = 0;
unsigned char c = 0;
unsigned char us = 0;
unsigned char tmp = 0x00;
unsigned char readberker = 0;
unsigned char lastsend = 0;
void User_Process(void)
{

if(mUSBUSARTIsTxTrfReady()){
	if(readberker > 0) {
		lastsend = 1;
		b=1;
		c=0;
		a=0;
		while(PORTBbits.RB0); 	//wait Low
		while(!PORTBbits.RB0); 	//wait Hi
  		Delay1uS(IR_611us);		//wait 611us
		if(PORTBbits.RB0 = 0) return;
  		Delay1uS(IR_611us);		//wait 611us
		if(PORTBbits.RB0 = 0) return;

		//while(PORTBbits.RB0); 	//wait Low
		while(1) {
			if(c >= 49 ) break;
			if(lastsend != PORTBbits.RB0){ //wait Change
				a=PORTBbits.RB0;
			    Delay1uS(IR_611us);		//wait 611us
				if(a != PORTBbits.RB0) { 
					output_buffer[c] = 0x31; //1
				} else {
					output_buffer[c] = 0x30; //0
				}
				lastsend = PORTBbits.RB0;
				c++;
			}
		}		
		output_buffer[c] = 0x0d; c++;
		output_buffer[c] = 0x0a; c++;
		output_buffer[c] = 0x00;
		mUSBUSARTTxRam((byte*)output_buffer,c); 
		readberker--;
		while(PORTBbits.RB0); 	//wait Low
 		Delay1uS(80000);		//wait 1ms	
		while(PORTBbits.RB0); 	//wait Low
		return;
	}
}

 if(getsUSBUSART(input_buffer,64))
    {
	  switch (input_buffer[0])
	  {
		//	case '1': if(input_buffer[1] == '1') { runad = 1; runioi = 0; } else { runad = 0;  } break;
		//	case '2': if(input_buffer[1] == '1') { runioi = 1; runad = 0; } else { runioi = 0; } break;
			case '1':	if(mUSBUSARTIsTxTrfReady()) usb_write_char_hex(PORTB); break;
			case '2':	if(mUSBUSARTIsTxTrfReady()) {
							ADCON0bits.GO = 1; // Start conversion
							while( ADCON0bits.GO ); // Wait for completion
							result = (((unsigned int)ADRESH)<<8)|(ADRESL); // Read result
							usb_write_int_hex(result); // send!
						} break;
			case '4': if(input_buffer[1] == '1') INTCON2bits.RBPU=0; else INTCON2bits.RBPU=1; break;
			case '3': 
				if(input_buffer[1] == '1')  LATDbits.LATD0 = 1; else LATDbits.LATD0 = 0;
				if(input_buffer[2] == '1')  LATDbits.LATD1 = 1; else LATDbits.LATD1 = 0;
				if(input_buffer[3] == '1')  LATDbits.LATD2 = 1; else LATDbits.LATD2 = 0;
				if(input_buffer[4] == '1')  LATDbits.LATD3 = 1; else LATDbits.LATD3 = 0;
				if(input_buffer[5] == '1')  LATDbits.LATD4 = 1; else LATDbits.LATD4 = 0;
				if(input_buffer[6] == '1')  LATDbits.LATD5 = 1; else LATDbits.LATD5 = 0;
				if(input_buffer[7] == '1')  LATDbits.LATD6 = 1; else LATDbits.LATD6 = 0;
				if(input_buffer[8] == '1')  LATDbits.LATD7 = 1; else LATDbits.LATD7 = 0;
			 break;

			case '5': //IR: bin
				us = input_buffer[2];
				//us	= tmp*10;
				for(i=0;i<input_buffer[1];i++){
					tmp = input_buffer[i+3];
					for(a=0;a<8;a++){
						if((tmp & 0x80) !=0){
 						//	LATDbits.LATD4 = 1; 
							SetDCPWM1(512);
					}	else {
		 				//	LATDbits.LATD4 = 0; 
							SetDCPWM1(0);
							}
						tmp = tmp << 1;
					b= 0; 
						while(1) { 
							if(b==us) break; 
							b++; 
							_asm
								nop nop nop nop     
								nop nop nop nop 
								nop nop nop nop     
								nop nop nop nop 
								
								nop nop nop nop     
								nop nop nop nop 
								nop nop nop nop     
								nop nop nop nop 

								nop nop nop nop     
								nop nop nop nop 
								nop nop nop nop     
								nop nop nop nop 
							_endasm 
						 }
						b = 0;
						while(1) { 
							if(b==us) break; 
							b++; 
							_asm
								nop nop nop nop     
								nop nop nop nop 
								nop nop nop nop     
								nop nop nop nop 
								
								nop nop nop nop     
								nop nop nop nop 
								nop nop nop nop     
								nop nop nop nop 

								nop nop nop nop     
								nop nop nop nop 
								nop nop nop nop     
								nop nop nop nop 
							_endasm 
						 }

					}
				}
				SetDCPWM1(0);
   			 break;



			case '6': //IR: RC-MM
				//header
				SetDCPWM1(IR_PWMDC_OM);
				Delay1uS(IR_416us);
				SetDCPWM1(0);
				Delay1uS(IR_277us);
				b = 0;
				for(i=0;i<input_buffer[1];i++){
					tmp = input_buffer[i+3];
				for(a=0;a<4;a++){
					switch (tmp & 0b11000000) {
					case 0b00000000: 
						SetDCPWM1(IR_PWMDC_OM);
						Delay1uS(IR_166us);
						SetDCPWM1(0);
						Delay1uS(IR_277us);
					break;
					case 0b01000000:
						SetDCPWM1(IR_PWMDC_OM);
						Delay1uS(IR_166us);
						SetDCPWM1(0);
						Delay1uS(IR_444us);
					break;
					case 0b10000000: 
						SetDCPWM1(IR_PWMDC_OM);
						Delay1uS(IR_166us);
						SetDCPWM1(0);
						Delay1uS(IR_611us);
					break;
					case 0b11000000: 
						SetDCPWM1(IR_PWMDC_OM);
						Delay1uS(IR_166us);
						SetDCPWM1(0);
						Delay1uS(IR_777us);
					break;
					}			
					tmp = tmp << 2; 
					b=b+1;
					if(b==input_buffer[2]) break;
				}
				if(b==input_buffer[2]) break;
			}
				//End
				SetDCPWM1(IR_PWMDC_OM);
				Delay1uS(IR_166us);
				SetDCPWM1(0);
   			 break;


		case '7': //AM: PL2292 / HX2292 (433MHz)
	for(c=0;c<4;c++){
				b = 0;
				for(i=0;i<input_buffer[1];i++){
					tmp = input_buffer[i+3];
				for(a=0;a<8;a++){
		


					if((tmp & 0x80) !=0){
						SetDCPWM2(IR_PWMDC_OM);
						Delay1uS(IR_350us);
						SetDCPWM2(0);
						Delay1uS(IR_1ms);

						SetDCPWM2(IR_PWMDC_OM);
						Delay1uS(IR_350us);
						SetDCPWM2(0);
						Delay1uS(IR_1ms);
				} else {
						SetDCPWM2(IR_PWMDC_OM);
						Delay1uS(IR_350us);
						SetDCPWM2(0);
						Delay1uS(IR_1ms);

						SetDCPWM2(IR_PWMDC_OM);
						Delay1uS(IR_1ms);
						SetDCPWM2(0);
						Delay1uS(IR_350us);
					}		

					tmp = tmp << 1; 
					b=b+1;
					if(b==input_buffer[2]) break;
				}
				if(b==input_buffer[2]) break;
			}
				 	SetDCPWM2(IR_PWMDC_OM); // LATDbits.LATD4 = 1; 
					Delay1uS(IR_350us);
					SetDCPWM2(0); // LATDbits.LATD4 = 0;	
					Delay1uS(9800);
}
	SetDCPWM2(0);
		break;
		case '8': //AM: Berker (433MHz)
	for(c=0;c<input_buffer[3];c++){
			lastsend = 1;
			SetDCPWM2(AM_PWMDC_OM);
			Delay1uS(IR_1ms);
			Delay1uS(IR_1ms);
			Delay1uS(IR_1ms);
			Delay1uS(IR_1ms);

			b = 0;
				for(i=0;i<input_buffer[1];i++){
					tmp = input_buffer[i+4];
				for(a=0;a<8;a++){
					if((tmp & 0x80) !=0){
						if(lastsend ==1) {
							SetDCPWM2(0);
							Delay1uS(IR_500us);
							SetDCPWM2(AM_PWMDC_OM);
							Delay1uS(IR_500us);
							lastsend = 1;
						} else {
							SetDCPWM2(AM_PWMDC_OM);
							Delay1uS(IR_500us);
							SetDCPWM2(0);
							Delay1uS(IR_500us);
							lastsend = 0;
						}
					} else {
						if(lastsend ==1) {
							SetDCPWM2(0);
							Delay1uS(IR_1ms);
							lastsend = 0;
						} else {
							SetDCPWM2(AM_PWMDC_OM);
							Delay1uS(IR_1ms);
							lastsend = 1;
						}
					}		
				
					tmp = tmp << 1; 
					b=b+1;
					if(b==input_buffer[2]) break;
				}
				if(b==input_buffer[2]) break;
			}
				 	SetDCPWM2(0); // LATDbits.LATD4 = 0;	
				//	Delay1uS(180000);
					Delay1uS(100000);
}
	SetDCPWM2(0);
		break;


	case 'i': 
readberker = 4;

break;

	case 'b':  USBModuleDisable(); UCONbits.USBEN = 0; Delay1uS(9800); EEDataW(0xFE , 0xFF);  Reset() break;
		  default   : 
						runad = 0;
						runioi = 0;
						if(mUSBUSARTIsTxTrfReady()) putrsUSBUSART("Links USB IO Interface!\n\r"); 
			break;
	  }  	 
    }    


/*
	if(mUSBUSARTIsTxTrfReady()) {
		if(runioi){	
			usb_write_char_hex(PORTB);
		}
		if(runad){
				ADCON0bits.GO = 1; // Start conversion
				while( ADCON0bits.GO ); // Wait for completion
				result = (((unsigned int)ADRESH)<<8)|(ADRESL); // Read result
				usb_write_int_hex(result); // send!
		}
	}
*/
 }//end 

   


/******************************************************************************
 * Function:        void BlinkUSBStatus(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        BlinkUSBStatus turns on and off LEDs corresponding to
 *                  the USB device state.
 *
 * Note:            mLED macros can be found in io_cfg.h
 *                  usb_device_state is declared in usbmmap.c and is modified
 *                  in usbdrv.c, usbctrltrf.c, and usb9.c
 *****************************************************************************/

void BlinkUSBStatus(void)
{
    static word led_count=0;
    
    if(led_count == 0)led_count = 10000U;
    led_count--;

    #define mLED_Both_Off()         {mLED_1_Off();mLED_2_Off();}
    #define mLED_Both_On()          {mLED_1_On();mLED_2_On();}
    #define mLED_Only_1_On()        {mLED_1_On();mLED_2_Off();}
    #define mLED_Only_2_On()        {mLED_1_Off();mLED_2_On();}

    if(UCONbits.SUSPND == 1)
    {
        if(led_count==0)
        {
            mLED_1_Toggle();
            mLED_2 = mLED_1;        // Both blink at the same time
        }//end if
    }
    else
    {
        if(usb_device_state == DETACHED_STATE)
        {
            mLED_Both_Off();
        }
        else if(usb_device_state == ATTACHED_STATE)
        {
            mLED_Both_On();
        }
        else if(usb_device_state == POWERED_STATE)
        {
            mLED_Only_1_On();
        }
        else if(usb_device_state == DEFAULT_STATE)
        {
            mLED_Only_2_On();
        }
        else if(usb_device_state == ADDRESS_STATE)
        {
            if(led_count == 0)
            {
                mLED_1_Toggle();
                mLED_2_Off();
            }//end if
        }
        else if(usb_device_state == CONFIGURED_STATE)
        {
            if(led_count==0)
            {
                mLED_1_Toggle();
             //   mLED_2 = !mLED_1;       // Alternate blink                
            }//end if
        }//end if(...)
    }//end if(UCONbits.SUSPND...)

}//end BlinkUSBStatus

